@@ -1,4 +1,6 @@
 package App::Tel;
+use strict;
+use warnings;
 
 =head1 NAME
 
@@ -6,7 +8,12 @@ App::Tel - A script for logging into devices
 
 =head1 VERSION
 
-0.2010_04
+0.2015_01
+
+=cut
+
+our $VERSION = eval  { 0.2015_01 };
+
 
 =head1 SYNOPSIS
 
@@ -28,8 +35,6 @@ under the same terms as Perl itself.
 
 =cut
 
-use strict;
-use warnings;
 use Expect qw( exp_continue );
 use IO::Stty;
 use POSIX qw(:sys_wait_h :unistd_h); # For WNOHANG and isatty
@@ -37,7 +42,6 @@ use Hash::Merge::Simple qw (merge);
 use Module::Load;
 use v5.10;
 
-our $VERSION = eval '0.2010_04';
 
 # For reasons related to state I needed to make $winch_it global
 # because it needs to be written to inside signals.
@@ -494,6 +498,10 @@ sub password {
     return 'undef';
 }
 
+sub _winch_handler {
+    $winch_it=1;
+}
+
 # Expect won't let you reuse a connection that has spawned a process, so you
 # can call this with an argument to reset the session.  If called without an
 # argument it will return the current session (If it exists) or create a new
@@ -509,11 +517,8 @@ sub session {
     $session = new Expect;
 
     # install sig handler for window size change
-    $SIG{WINCH} = \&winch_handler;
+    $SIG{WINCH} = \&_winch_handler;
 
-    sub winch_handler {
-        $winch_it=1;
-    }
 
     $session->log_stdout(1);
     $self->{'session'} = $session;
@@ -530,7 +535,7 @@ sub winch {
         kill WINCH => $session->pid if $session->pid;
     };
     $winch_it=0;
-    $SIG{WINCH} = \&winch_handler;
+    $SIG{WINCH} = \&_winch_handler;
 }
 
 # this sets up the session.  If there already is a session open it closes and
@@ -753,7 +758,6 @@ sub interconnect {
     my $self = shift;
 	my @handles = ($self->{'session'}, $_[0]);
 
-	#  my ($handle)=(shift); call as Expect::interconnect($spawn1,$spawn2,...)
 	my ( $nread );
 	my ( $rout, $emask, $eout );
 	my ( $escape_character_buffer );
@@ -830,7 +834,6 @@ sub interconnect {
 			}
 		}
 
-		# Every second? No, go until we get something from someone.
 		my $nfound = select( $rout = $read_mask, undef, $eout = $emask, undef );
 
         # Is there anything to share?  May be -1 if interrupted by a signal...
@@ -839,9 +842,8 @@ sub interconnect {
 
 		# Which handles have stuff?
 		my @bits = split( //, unpack( 'b*', $rout ) );
-		$eout = 0 unless defined($eout);
-		my @ebits = split( //, unpack( 'b*', $eout ) );
-
+		#$eout = 0 unless defined($eout);
+		#my @ebits = split( //, unpack( 'b*', $eout ) );
 		#    print "Ebits: $eout\r\n";
 		foreach my $read_handle (@handles) {
 			if ( $bits[ $read_handle->fileno() ] ) {
@@ -978,38 +980,46 @@ sub control_loop {
     my $opts = $self->{opts};
     my $prompt = $profile->{prompt};
     my $pagercmd = $profile->{pagercmd};
-    my $autocmds = [ split(/;/, $opts->{a}) ] if ($opts->{a});
-    $autocmds ||= $profile->{autocmds};
+    my $autocmds;
+    my @args;
+
+    if ($opts->{a}) {
+        $autocmds = [ split(/;/, $opts->{a}) ];
+    } else {
+        $autocmds = $profile->{autocmds};
+    }
 
     $self->winch();
 
-    my @args = split(/;/, $opts->{c}) if ($opts->{c});
+    # should -c override -x or be additive? or error if both are specified?
+
+    @args = split(/;/, $opts->{c}) if ($opts->{c});
+
     if ($opts->{x}) {
-        open(X, '<', $opts->{x});
-        @args = <X>;
-        close X;
+        open(my $X, '<', $opts->{x});
+        @args = <$X>;
+        close $X;
     }
 
     if (@args) {
-        $self->expect(10,'-re',$prompt);
+        $self->expect($self->{timeout},'-re',$prompt);
         if (ref($pagercmd) eq 'CODE') {
             $pagercmd->();
             undef $pagercmd;
-        } else {
-            $pagercmd ||= 'term len 0';
+        } elsif ($pagercmd) {
             $self->send("$pagercmd\r");
-            $self->expect(10,'-re',$prompt);
+            $self->expect($self->{timeout},'-re',$prompt);
         }
         foreach my $arg (@args) {
             chomp($arg);
             $self->send("$arg\r");
-            $self->expect(10,'-re',$prompt);
+            $self->expect($self->{timeout},'-re',$prompt);
         }
         $self->send($profile->{logoutcmd} ."\r");
     } else {
         foreach my $arg (@$autocmds) {
             $self->send("$arg\r");
-            $self->expect(10,'-re',$prompt);
+            $self->expect($self->{timeout},'-re',$prompt);
         }
         $self->interact($self->{stdin}, '\cD');
         # q\b is to end anything that's at a More prompt or other dialog and
@@ -1022,4 +1032,14 @@ sub control_loop {
     }
 }
 
+sub handle_backspace {
+    ${$_[0]}->send("\b");
+    return 1;
+}
+
+sub handle_ctrl_z {
+    ${$_[0]}->send("exit\r");
+    return 1;
+}
+
 1;
